program gtest
  implicit none

  real*8 :: d, dmin, dmax, g, gp, h, hp
  integer :: i, npts

  dmin = 1.0d0
  dmax = 10.0d0
  npts = 100001

  do i = 1, npts
     d = dmin + real(i-1,8) / real(npts-1,8) * (dmax-dmin)
     call calcgh_xdm(d,g,gp,h,hp)
     write (*,'(I6,X,1p,5(E20.12,X))') i, d, g, gp, h, hp
  end do

contains

  subroutine calcgh_xdm(d,g,gp,h,hp)
    implicit none
    real*8, intent(in) :: d
    real*8, intent(out) :: g, gp, h, hp

    real*8 :: c6, c8, c10, rvdw
    real*8 :: d2, d4, d6, d8, d10, dpr6, dpr8, dpr10, r2, r6, r8, r10
    real*8 :: d5, d7, d9, dpr6sq, dpr8sq, dpr10sq, d17, d13, d3, dpr6cub
    real*8 :: dpr8cub, dpr10cub

    ! graphite
    c6 = 1.771066d+01 * 2d0 ! to ry
    c8 = 6.829096d+02 * 2d0 ! to ry
    c10 = 2.876529d+04 * 2d0 ! to ry
    rvdw = 7.308851d0

    r2 = rvdw * rvdw
    r6 = r2 * r2 * r2
    r8 = r6 * r2
    r10 = r8 * r2

    d2 = d * d
    d3 = d2 * d
    d4 = d2 * d2
    d5 = d4 * d
    d6 = d4 * d2
    d7 = d6 * d
    d8 = d6 * d2
    d9 = d8 * d
    d10 = d8 * d2
    d13 = d6 * d7
    d17 = d10 * d7

    dpr6 = r6 + d6
    dpr8 = r8 + d8
    dpr10 = r10 + d10
    dpr6sq = dpr6 * dpr6
    dpr8sq = dpr8 * dpr8
    dpr10sq = dpr10 * dpr10
    dpr6cub = dpr6sq * dpr6
    dpr8cub = dpr8sq * dpr8
    dpr10cub = dpr10sq * dpr10

    g = c6 / dpr6 + c8 / dpr8 + c10 / dpr10
    gp = -10d0 * c10 * d9 / dpr10sq - 8d0 * c8 * d7 / dpr8sq - 6d0 * c6 * d5 / dpr6sq
    h = gp / d
    hp = -80d0 * c10 * d7 / dpr10sq + 200d0 * c10 * d17 / dpr10cub - 48d0 * c8 * d5 / dpr8sq &
       + 128d0 * c8 * d13 / dpr8cub - 24 * c6 * d3 / dpr6sq + 72d0 * c6 * d9 / dpr6cub

  end subroutine calcgh_xdm

  subroutine calcgh_d2(d,g,gp,h,hp)
    implicit none
    real*8, intent(in) :: d
    real*8, intent(out) :: g, gp, h, hp

    real*8 :: ed, fij, d6, d7, d2, scal6, c6_ij, r_sum, beta

    ! graphite
    scal6 = 0.75d0
    c6_ij = 60.71d0
    r_sum = 5.488d0
    beta = 20d0

    d2 = d * d
    d6 = d**6
    d7 = d6 * d
    ed = exp(-beta * (d / r_sum - 1.d0))
    fij = 1.d0 / (1.d0 + ed)
    g = c6_ij * scal6 / d6 * fij
    gp = c6_ij * scal6 / d6 / (1.d0 + ed) * (beta * ed / r_sum / (1.d0 + ed) - 6.d0 / d)
    h = gp / d
    hp = c6_ij * scal6 / d7 / (1.d0 + ed) * (48.d0 / d2 - &
       13.d0 * beta * ed / r_sum / d / (1.d0 + ed) - &
       beta**2 * ed / r_sum**2 / (1.d0 + ed)**2 * (1.d0 - ed))

  end subroutine calcgh_d2

end program gtest

